@@ -33,6 +33,7 @@ import itertools
 import warnings
 import logging
 import traceback
+import json
 
 
 from .compat import (
@@ -65,6 +66,7 @@ from .errors import HTTPrettyError
 
 from datetime import datetime
 from datetime import timedelta
+from errno import EAGAIN
 
 old_socket = socket.socket
 old_create_connection = socket.create_connection
@@ -94,20 +96,21 @@ except ImportError:
     ssl = None
 
 
-POTENTIAL_HTTP_PORTS = [80, 443]
+POTENTIAL_HTTP_PORTS = set([80, 443])
 
 
 class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
     def __init__(self, headers, body=''):
         self.body = utf8(body)
         self.raw_headers = utf8(headers)
-        self.rfile = StringIO(b'\r\n\r\n'.join([headers.strip(), body]))
+        self.rfile = StringIO(b'\r\n\r\n'.join([utf8(headers.strip()), self.body]))
         self.wfile = StringIO()
         self.raw_requestline = self.rfile.readline()
         self.error_code = self.error_message = None
         self.parse_request()
         self.method = self.command
         self.querystring = parse_qs(self.path.split("?", 1)[-1])
+        self.parsed_body = self.__parse_body(self.body)
 
     def __str__(self):
         return 'HTTPrettyRequest(headers={0}, body="{1}")'.format(
@@ -115,6 +118,19 @@ class HTTPrettyRequest(BaseHTTPRequestHandler, BaseClass):
             self.body,
         )
 
+    def __parse_body(self, body):
+        """ Attempt to parse the post based on the content-type passed. Return the regular body if not """
+        return_value = body.decode('utf-8')
+        try:
+            for header in self.headers.keys():
+                if header.lower() == 'content-type':
+                    if self.headers['content-type'].lower() == 'application/json':
+                        return_value = json.loads(return_value)
+                    elif self.headers['content-type'].lower() == 'application/x-www-form-urlencoded':
+                        return_value = parse_qs(return_value)
+        finally:
+            return return_value
+
 
 class EmptyRequestHeaders(dict):
     pass
@@ -221,8 +237,9 @@ class fakesock(object):
                     _d = self.truesock.recv(16)
                     self.truesock.settimeout(0.0)
                     self.fd.write(_d)
-
-                except socket.error:
+                except socket.error as e:
+                    if e.errno == EAGAIN:
+                        continue
                     break
 
             self.fd.seek(0)
@@ -263,7 +280,7 @@ class fakesock(object):
 
             # path might come with
             s = urlsplit(path)
-            POTENTIAL_HTTP_PORTS.append(int(s.port or 80))
+            POTENTIAL_HTTP_PORTS.add(int(s.port or 80))
             headers, body = map(utf8, data.split(b'\r\n\r\n', 1))
 
             request = httpretty.historify_request(headers, body)
@@ -349,7 +366,7 @@ def create_fake_connection(address, timeout=socket._GLOBAL_DEFAULT_TIMEOUT, sour
 
 
 def fake_gethostbyname(host):
-    return host
+    return '127.0.0.1'
 
 
 def fake_gethostname():
@@ -553,12 +570,12 @@ class URIInfo(BaseClass):
     def __eq__(self, other):
         self_tuple = (
             self.port,
-            decode_utf8(self.hostname),
+            decode_utf8(self.hostname.lower()),
             url_fix(decode_utf8(self.path)),
         )
         other_tuple = (
             other.port,
-            decode_utf8(other.hostname),
+            decode_utf8(other.hostname.lower()),
             url_fix(decode_utf8(other.path)),
         )
         return self_tuple == other_tuple
@@ -585,7 +602,7 @@ class URIInfo(BaseClass):
     @classmethod
     def from_uri(cls, uri, entry):
         result = urlsplit(uri)
-        POTENTIAL_HTTP_PORTS.append(int(result.port or 80))
+        POTENTIAL_HTTP_PORTS.add(int(result.port or 80))
         return cls(result.username,
                    result.password,
                    result.hostname,
@@ -601,7 +618,8 @@ class URIMatcher(object):
     regex = None
     info = None
 
-    def __init__(self, uri, entries):
+    def __init__(self, uri, entries, match_querystring=False):
+        self._match_querystring = match_querystring
         if type(uri).__name__ == 'SRE_Pattern':
             self.regex = uri
         else:
@@ -616,7 +634,8 @@ class URIMatcher(object):
         if self.info:
             return self.info == info
         else:
-            return self.regex.search(info.full_url(use_querystring=False))
+            return self.regex.search(info.full_url(
+                use_querystring=self._match_querystring))
 
     def __str__(self):
         wrap = 'URLMatcher({0})'
@@ -683,7 +702,8 @@ class httpretty(HttpBaseClass):
                      adding_headers=None,
                      forcing_headers=None,
                      status=200,
-                     responses=None, **headers):
+                     responses=None, match_querystring=False,
+                     **headers):
 
         uri_is_string = isinstance(uri, basestring)
 
@@ -705,7 +725,8 @@ class httpretty(HttpBaseClass):
                 cls.Response(method=method, uri=uri, **headers),
             ]
 
-        matcher = URIMatcher(uri, entries_for_this_uri)
+        matcher = URIMatcher(uri, entries_for_this_uri,
+                             match_querystring)
         if matcher in cls._entries:
             matcher.entries.extend(cls._entries[matcher])
             del cls._entries[matcher]
